use item_mapping::ItemMapping;
use seed_range::SeedRange;

mod item_mapping;
mod parsing;
mod seed_range;
type UnsignedNumber = u128;

pub fn solve_task_1(input: &str) -> String {
    let (seeds, mapping_layers) = parsing::parse(input);
    let min = min_location_from_given_single_seeds(&seeds, &mapping_layers);
    min.to_string()
}

fn min_location_from_given_single_seeds(
    seeds: &[UnsignedNumber],
    layers: &[Vec<ItemMapping>],
) -> UnsignedNumber {
    let mut min = UnsignedNumber::MAX;
    for next_seed in seeds {
        let mut current_seed = *next_seed;
        for next_layer in layers {
            for next_mapping in next_layer {
                match next_mapping.map_point(current_seed) {
                    Some(found_mapping) => {
                        current_seed = found_mapping;
                        break;
                    }
                    None => (),
                }
            }
        }
        min = current_seed.min(min);
    }
    min
}

pub fn solve_task_2(input: &str) -> String {
    let (starting_from, mappings) = {
        let (single_seeds, mappings) = parsing::parse(input);
        let seed_ranges = combine_single_to_ranges_for_seeds(&single_seeds);
        (seed_ranges, mappings)
    };
    "".to_string()
}

fn combine_single_to_ranges_for_seeds(single_seeds: &[UnsignedNumber]) -> Vec<SeedRange> {
    assert!(single_seeds.len() % 2 == 0);
    single_seeds
        .chunks(2)
        .into_iter()
        .map(|array| match array {
            [start, end] => {
                let start = *start;
                let range = end - 1;
                let end = start + range;
                SeedRange::new(start, end)
            }
            _ => unreachable!(),
        })
        .collect()
}

#[cfg(test)]
mod testing {
    use crate::solutions::day5::{
        combine_single_to_ranges_for_seeds, parsing, seed_range::SeedRange, solve_task_2,
    };

    use super::solve_task_1;
    const EXAMPLE_INPUT: &str = include_str!("day5/day_example_input.txt");

    #[test]
    fn should_solve_task_1() {
        let actual = solve_task_1(EXAMPLE_INPUT);
        let expected = "35";
        assert_eq!(expected, actual)
    }

    #[test]
    fn should_combine_single_to_ranges_for_seeds() {
        let (single_seeds, _) = parsing::parse(EXAMPLE_INPUT);
        let actual = combine_single_to_ranges_for_seeds(&single_seeds);
        let expected = vec![SeedRange::new(79, 92), SeedRange::new(55, 67)];
        assert_eq!(expected, actual)
    }

    #[test]
    fn should_solve_task2() {
        let actual = solve_task_2(EXAMPLE_INPUT);
        let expected = "46";
        assert_eq!(expected, actual)
    }
}
